(* $Id: menu_aux.ml 63 2003-08-24 15:39:25Z gerd $
 * ----------------------------------------------------------------------
 * Copyright 2003 by Gerd Stolpmann. 
 * 
 * This file is part of godi-tools. 
 *
 * godi-tools is free software; you can redistribute it and/or modify it
 * under the conditions in the file LICENSE.
 *
 * godi-tools is distributed WITHOUT ANY WARRANTY, as detailed in the
 * file LICENSE.
 *)


let overwrite_centered line s =
  let l_line = String.length line in
  let l_s = String.length s in
  if l_s >= l_line then
    s
  else (
    let line' = String.copy line in
    let p = l_line / 2 - l_s / 2 in
    String.blit s 0 line' p l_s;
    line'
  )
;;
    

let add_full_head ?(lmargin=10) (pg : Miniui.page) menuname =
  pg # configure ~head_region_height: 6 ();

  let w = pg # width() in

  let minus_98 = String.make (w-2) '-' in
  let spaces_98 = String.make (w-2) ' ' in

  let put n = pg # put_line ~region:`Head ~line:(`Replace n) in

  put 0 ("+" ^ minus_98 ^ "+");
  put 1 (overwrite_centered ("|" ^ spaces_98 ^ "|") "GODI Console");
  put 2 ("+" ^ minus_98 ^ "+");
  put 3 "";
  put 4 ~lmargin menuname;
  put 5 "";

  ()
;;
    

let add_compact_head ?(lmargin=10) (pg : Miniui.page) menuname =
  pg # configure ~head_region_height: 4 ();

  let w = pg # width() in

  let minus_98 = String.make (w-2) '-' in
  let spaces_98 = String.make (w-2) ' ' in

  let put n = pg # put_line ~region:`Head ~line:(`Replace n) in

  put 0 (overwrite_centered ("+" ^ minus_98 ^ "+") " GODI Console ");
  put 1 "";
  put 2 ~lmargin menuname;
  put 3 "";

  ()
;;


type col_spec =
  [ `Left of int
  | `Right of int
  | `Delim of string
  ]
(* TODO: `Center *)
;;


let mk_formatted_line (columns:col_spec list) (data:string list) =
  let cols = ref columns in
  let line = ref "" in

  let have_delim() =
    match !cols with
	`Delim _ :: _ -> true
      | _ -> false
  in

  let get_delim() =
    match !cols with
	`Delim d :: _ -> d
      | _ -> ""
  in

  List.iter
    (fun datum ->
       while have_delim() do
	 line := !line ^ (get_delim());
	 cols := List.tl !cols;
       done;
       ( match !cols with
	     col :: cols' ->
	       ( match col with
		     `Left n ->
		       let m = min n (String.length datum) in
		       let s =
			 if String.length datum > n then
			   String.sub datum 0 (n-1) ^ "$"
			 else
			   datum ^ String.make (n-m) ' ' in
		       line := !line ^ s
		   | `Right n ->
		       let m = min n (String.length datum) in
		       let s =
			 if String.length datum > n then
			   String.sub datum 0 (n-1) ^ "$"
			 else
			   String.make (n-m) ' ' ^ datum in
		       line := !line ^ s
		   | `Delim d ->
		       assert false
	       );
	       cols := cols'
	   | [] ->
	       line := !line ^ datum
       )
    )
    data;

  !line
;;


type menu_item =
  [ `Head of (string list)
  | `Item of (string list * int * (unit -> unit))
  | `Head_separator of string
  | `Separator of string
  ]
;;

let add_menu 
       ?lmargin ?rmargin
       (pg : Miniui.page) (columns:col_spec list) (items:menu_item list)=
  List.iter
    (fun item ->
       match item with
	   `Head data ->
	     let n = pg # height ~region:`Head () in
	     pg # put_line ~region:`Head ~line:(`Replace n) ?lmargin ?rmargin 
	                   (mk_formatted_line columns data);
	     pg # configure ~head_region_height:(n+1) ();
	 | `Head_separator sep ->
	     let n = pg # height ~region:`Head () in
	     pg # put_line ~region:`Head ~line:(`Replace n) ?lmargin ?rmargin
                           sep;
	     pg # configure ~head_region_height:(n+1) ();
	 | `Item (data, decimal, callback) ->
	     let s = mk_formatted_line columns data in
	     pg # put_line ?lmargin ?rmargin s;
	     let line = pg # height() - 1 in
	     let _ =
	       pg # add_body_button ~line ~left:0 ~width:(String.length s)
                                    ~callback (`Number decimal) in
	     ()
	 | `Separator sep ->
	     pg # put_line ?lmargin ?rmargin sep
    )
    items
;;
