(* $Id: godi_perform.ml 177 2003-09-01 14:31:19Z gerd $
 * ----------------------------------------------------------------------
 * Copyright 2003 by Gerd Stolpmann.
 *
 * This file is part of godi-tools.
 *
 * godi-tools is free software; you can redistribute it and/or modify it
 * under the conditions in the file LICENSE.
 *
 * godi-tools is distributed WITHOUT ANY WARRANTY, as detailed in the
 * file LICENSE.
 *)

open Godi_types


let remove_pkg ?interactivity ?protocol pkg =
  if pkgtype pkg <> `Installed then
    failwith "Godi_perform.remove_pkg: not an installed package";
  Godi_exec.call_cmd
    ?stdout:protocol
    ?stderr:protocol
    ?interactivity
    ("godi_delete", [| pkg_name pkg |] )
;;


let add_pkg ?interactivity ?protocol pkg =
  if pkgtype pkg <> `Binary then
    failwith "Godi_perform.add_pkg: not a binary package";
  Godi_exec.call_cmd
    ?stdout:protocol
    ?stderr:protocol
    ?interactivity
    ("godi_add", [| (binary_pkg pkg)#binary_archive |] )
;;


let test_conf_pkg ?interactivity ?protocol
                  ?(godi_conf = Godi_query.godi_conf()) pkg =
  if pkgtype pkg <> `Source then
    failwith "Godi_perform.test_conf_pkg: not a source package";
  let dir = (source_pkg pkg) # source_directory in
  Godi_exec.call_cmd
    ~cd:dir
    ?stdout:protocol
    ?stderr:protocol
    ?interactivity
    ("godi_make", [| "clean" |] );
  (* Classic source package *)
  Godi_exec.call_cmd
    ~cd:dir
    ~env:[ "GODI_CONF", godi_conf ]
    ?stdout:protocol
    ?stderr:protocol
    ?interactivity
    ("godi_make", [| "configure" |] );
  Godi_exec.call_cmd
    ~cd:dir
    ?stdout:protocol
    ?stderr:protocol
    ?interactivity
    ("godi_make", [| "clean" |] )
;;


let build_pkg ?interactivity ?protocol pkg =
  if pkgtype pkg <> `Source then
    failwith "Godi_perform.build_pkg: not a source package";
  let dir = (source_pkg pkg) # source_directory in
  Godi_exec.call_cmd
    ~cd:dir
    ?stdout:protocol
    ?stderr:protocol
    ?interactivity
    ("godi_make", [| "clean" |] );
  if (source_pkg pkg)#self_update_pkg <> "" then (
    (* Package to self-update GODI: *)
    Godi_exec.call_cmd
      ~cd:dir
      ?stdout:protocol
      ?stderr:protocol
      ?interactivity
      ("godi_make", [| "self_package" |] );
    (* Force deletion, ignore errors: *)
    ( try
	Godi_exec.call_cmd
	  ?stdout:protocol
	  ?stderr:protocol
	  ?interactivity
	  ("godi_delete", [| "-f"; (source_pkg pkg)#name |] )
      with
	  Godi_exec.Command_failure(_,_,_) ->
	    ()
    );
    Godi_exec.call_cmd
      ?stdout:protocol
      ?stderr:protocol
      ?interactivity
      ("godi_add", [| (source_pkg pkg)#self_update_pkg |] )
  ) else (
    (* Classic source package *)
    Godi_exec.call_cmd
	    ~cd:dir
	    ?stdout:protocol
	    ?stderr:protocol
	    ?interactivity
	    ("godi_make", [| "package" |] );
  );
  Godi_exec.call_cmd
    ~cd:dir
    ?stdout:protocol
    ?stderr:protocol
    ?interactivity
    ("godi_make", [| "clean" |] )
;;

